function [GIRF,GIRFopt]=simGIRF(IVAR, state, h, historyindex, impulse, fast, printprogress)

% references Koop, Peseran & Potter (1996), Giovanni Pellegrino
% OIRFhist(t,j,k,hist) : GIRFs associated to each starting history in the input vector "histories"

%% Unpack and re-define settings
c_case = IVAR.c_case;
n      = IVAR.n;
p      = IVAR.p;
A      = IVAR.A;
S      = IVAR.S;
uu     = IVAR.u;
historyindex = logical([0; historyindex(1:end-1)]); % lag by 1 period
histories = [IVAR.X, IVAR.Xex];
if c_case == 1
    histories = [ones(size(histories,1), 1), histories];
elseif c_case == 2
    histories = [ones(size(histories,1), 1), 1:1:size(histories,1), histories];
end
histories = histories(historyindex,:);
nhist  = size(histories,1);
npx    = size(histories, 2); % number of coefficients for each equation: = c_case + n*p (endogenous lags) + p (exogenous lags)

shockpos = state.shockpos;
statepos = state.statepos;

iter = 500; % number of repetitions simulated for each initial condition
if fast==1
    iter=2; %no extraction of future shocks, 2 draws sufficient
end
mode = 0; % 0 = draw from empirical distribution of residuals

nexplos=0; % to count for the number of explosive histories
nrep=zeros(1,nhist);
IRFhist=zeros(h, n, nhist);
if printprogress
    disp('Progress \n')
end

%% Loop over all histories defined in input
for ii=1:nhist % if parallel toolbox is installed, replace 'for' with 'parfor'
    
    if printprogress  % progress bar
        fprintf(1,'\b\b\b\b%3.0f%%',100*(ii/nhist)); 
    end
    
    irf = zeros(h,n,iter); % lower case irf: h x n x draws (500)
        
    ic=1;
    while ic <= iter

        % preparations, containers
        u = zeros(h,n);
        eps = zeros(1,n);       
        ushocked = zeros(1,n);
        X=histories(ii,:);  %predefined matrix for the starting values for the RHS variables
        W=histories(ii,:);
        Y=zeros(h,n); % variables including shock
        Z=zeros(h,n); % variable excluding shock
                  
        % settings
        if fast==0
            if mode==0
                u = uu(ceil(size(uu,1)*rand(h,1)),:); % for each h of horizon and variable, draw shock from empirical distribution
            else
                u = randn(h,n)*S'  ; % extract from a multivariate normal distribution respecting the VCV matrix (seen from the help for the function "randn")
            end
        end
        eps(1,:)= (S\u(1,:)')' ; % orthogonalized shock
        
        % add an additional to the structural eps vector   
        if impulse==0
            shock=1 ;
            eps(1,shockpos)=eps(1,shockpos)+shock;
        else
            shock=(impulse./S(shockpos,shockpos)) ; %initial shock (in terms of st.dev.)
            eps(1,shockpos)=eps(1,shockpos)+shock;
        end
        ushocked(1,:)=S*eps(1,:)'; % re-transform to reduced-form residual  
        
        % period 0
        Y(1,:)=X*A+ushocked(1,:);
        Z(1,:)=W*A+u(1,:); % only usual residuals
        irf(1,:,ic)=Y(1,:)-Z(1,:) ; % response in the period 0
        
        % period 1 to h-1
        for tt=1:h-1
                
            NewX=zeros(1,npx);
            NewW=zeros(1,npx);
            if c_case==2 % if c_case = 2, then RHS includes the time
                time=histories(ii,2);
                NewX(1,1:2)=[1,time+tt];
                NewW(1,1:2)=[1,time+tt];
            elseif c_case==1
                NewX(1,1)=1;
                NewW(1,1)=1;                   
            end
            
            % the amount by which X (and W) are 'moved' to the left every iteration depends on the number of lags (p) 
            if p == 1
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
            elseif p == 2
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
            elseif p == 3
                NewX(1,1+c_case+2*n:c_case+3*n)=X(1,1+c_case+n:c_case+2*n);
                NewW(1,1+c_case+2*n:c_case+3*n)=W(1,1+c_case+n:c_case+2*n);
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
            elseif p == 4
                NewX(1,1+c_case+3*n:c_case+4*n)=X(1,1+c_case+2*n:c_case+3*n);
                NewW(1,1+c_case+3*n:c_case+4*n)=W(1,1+c_case+2*n:c_case+3*n);
                NewX(1,1+c_case+2*n:c_case+3*n)=X(1,1+c_case+n:c_case+2*n);
                NewW(1,1+c_case+2*n:c_case+3*n)=W(1,1+c_case+n:c_case+2*n);
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
            elseif p == 5
                NewX(1,1+c_case+4*n:c_case+5*n)=X(1,1+c_case+3*n:c_case+4*n);
                NewW(1,1+c_case+4*n:c_case+5*n)=W(1,1+c_case+3*n:c_case+4*n);
                NewX(1,1+c_case+3*n:c_case+4*n)=X(1,1+c_case+2*n:c_case+3*n);
                NewW(1,1+c_case+3*n:c_case+4*n)=W(1,1+c_case+2*n:c_case+3*n);
                NewX(1,1+c_case+2*n:c_case+3*n)=X(1,1+c_case+n:c_case+2*n);
                NewW(1,1+c_case+2*n:c_case+3*n)=W(1,1+c_case+n:c_case+2*n);
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
            elseif p == 6
                NewX(1,1+c_case+5*n:c_case+6*n)=X(1,1+c_case+4*n:c_case+5*n);
                NewW(1,1+c_case+5*n:c_case+6*n)=W(1,1+c_case+4*n:c_case+5*n);
                NewX(1,1+c_case+4*n:c_case+5*n)=X(1,1+c_case+3*n:c_case+4*n);
                NewW(1,1+c_case+4*n:c_case+5*n)=W(1,1+c_case+3*n:c_case+4*n);
                NewX(1,1+c_case+3*n:c_case+4*n)=X(1,1+c_case+2*n:c_case+3*n);
                NewW(1,1+c_case+3*n:c_case+4*n)=W(1,1+c_case+2*n:c_case+3*n);
                NewX(1,1+c_case+2*n:c_case+3*n)=X(1,1+c_case+n:c_case+2*n);
                NewW(1,1+c_case+2*n:c_case+3*n)=W(1,1+c_case+n:c_case+2*n);
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
            elseif p == 7
                NewX(1,1+c_case+6*n:c_case+7*n)=X(1,1+c_case+5*n:c_case+6*n);
                NewW(1,1+c_case+6*n:c_case+7*n)=W(1,1+c_case+5*n:c_case+6*n);
                NewX(1,1+c_case+5*n:c_case+6*n)=X(1,1+c_case+4*n:c_case+5*n);
                NewW(1,1+c_case+5*n:c_case+6*n)=W(1,1+c_case+4*n:c_case+5*n);
                NewX(1,1+c_case+4*n:c_case+5*n)=X(1,1+c_case+3*n:c_case+4*n);
                NewW(1,1+c_case+4*n:c_case+5*n)=W(1,1+c_case+3*n:c_case+4*n);
                NewX(1,1+c_case+3*n:c_case+4*n)=X(1,1+c_case+2*n:c_case+3*n);
                NewW(1,1+c_case+3*n:c_case+4*n)=W(1,1+c_case+2*n:c_case+3*n);
                NewX(1,1+c_case+2*n:c_case+3*n)=X(1,1+c_case+n:c_case+2*n);
                NewW(1,1+c_case+2*n:c_case+3*n)=W(1,1+c_case+n:c_case+2*n);
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
             elseif p == 8
                NewX(1,1+c_case+7*n:c_case+8*n)=X(1,1+c_case+6*n:c_case+7*n);
                NewW(1,1+c_case+7*n:c_case+8*n)=W(1,1+c_case+6*n:c_case+7*n);
                NewX(1,1+c_case+6*n:c_case+7*n)=X(1,1+c_case+5*n:c_case+6*n);
                NewW(1,1+c_case+6*n:c_case+7*n)=W(1,1+c_case+5*n:c_case+6*n);
                NewX(1,1+c_case+5*n:c_case+6*n)=X(1,1+c_case+4*n:c_case+5*n);
                NewW(1,1+c_case+5*n:c_case+6*n)=W(1,1+c_case+4*n:c_case+5*n);
                NewX(1,1+c_case+4*n:c_case+5*n)=X(1,1+c_case+3*n:c_case+4*n);
                NewW(1,1+c_case+4*n:c_case+5*n)=W(1,1+c_case+3*n:c_case+4*n);
                NewX(1,1+c_case+3*n:c_case+4*n)=X(1,1+c_case+2*n:c_case+3*n);
                NewW(1,1+c_case+3*n:c_case+4*n)=W(1,1+c_case+2*n:c_case+3*n);
                NewX(1,1+c_case+2*n:c_case+3*n)=X(1,1+c_case+n:c_case+2*n);
                NewW(1,1+c_case+2*n:c_case+3*n)=W(1,1+c_case+n:c_case+2*n);
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
             elseif p == 9
                NewX(1,1+c_case+8*n:c_case+9*n)=X(1,1+c_case+7*n:c_case+8*n);
                NewW(1,1+c_case+8*n:c_case+9*n)=W(1,1+c_case+7*n:c_case+8*n);
                NewX(1,1+c_case+7*n:c_case+8*n)=X(1,1+c_case+6*n:c_case+7*n);
                NewW(1,1+c_case+7*n:c_case+8*n)=W(1,1+c_case+6*n:c_case+7*n);
                NewX(1,1+c_case+6*n:c_case+7*n)=X(1,1+c_case+5*n:c_case+6*n);
                NewW(1,1+c_case+6*n:c_case+7*n)=W(1,1+c_case+5*n:c_case+6*n);
                NewX(1,1+c_case+5*n:c_case+6*n)=X(1,1+c_case+4*n:c_case+5*n);
                NewW(1,1+c_case+5*n:c_case+6*n)=W(1,1+c_case+4*n:c_case+5*n);
                NewX(1,1+c_case+4*n:c_case+5*n)=X(1,1+c_case+3*n:c_case+4*n);
                NewW(1,1+c_case+4*n:c_case+5*n)=W(1,1+c_case+3*n:c_case+4*n);
                NewX(1,1+c_case+3*n:c_case+4*n)=X(1,1+c_case+2*n:c_case+3*n);
                NewW(1,1+c_case+3*n:c_case+4*n)=W(1,1+c_case+2*n:c_case+3*n);
                NewX(1,1+c_case+2*n:c_case+3*n)=X(1,1+c_case+n:c_case+2*n);
                NewW(1,1+c_case+2*n:c_case+3*n)=W(1,1+c_case+n:c_case+2*n);
                NewX(1,1+c_case+n:c_case+2*n)=X(1,1+c_case:c_case+n);
                NewW(1,1+c_case+n:c_case+2*n)=W(1,1+c_case:c_case+n);
                NewX(1,1+c_case:c_case+n)=Y(tt,:) ;
                NewW(1,1+c_case:c_case+n)=Z(tt,:);
            else
                error('GIRFs only available up to p = 9. Please write code forward if more are needed.')
            end
            
            % Based on the re-computed endogenous variables, we re-compute
            % the 'exogenous' interactions
            if statepos~=0
                if p==1
                    ordint=c_case+shockpos;
                    ordint2=c_case+statepos;
                    NewX(1,npx)=NewX(1,ordint2)*NewX(1,ordint);
                    NewW(1,npx)=NewW(1,ordint2)*NewW(1,ordint);
                elseif p==2
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos];
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos];
                    NewX(1,npx-1)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewW(1,npx-1)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                elseif p==3
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos,c_case+2*n+shockpos];  
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos,c_case+2*n+statepos];
                    NewX(1,npx-2)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx-1)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewX(1,npx)=NewX(1,ordint2(1,3))*NewX(1,ordint(1,3));
                    NewW(1,npx-2)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx-1)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                    NewW(1,npx)=NewW(1,ordint2(1,3))*NewW(1,ordint(1,3));
                elseif p==4
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos,c_case+2*n+shockpos,c_case+3*n+shockpos]; 
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos,c_case+2*n+statepos,c_case+3*n+statepos];
                    NewX(1,npx-3)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx-2)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewX(1,npx-1)=NewX(1,ordint2(1,3))*NewX(1,ordint(1,3));
                    NewX(1,npx)=NewX(1,ordint2(1,4))*NewX(1,ordint(1,4));
                    NewW(1,npx-3)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx-2)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                    NewW(1,npx-1)=NewW(1,ordint2(1,3))*NewW(1,ordint(1,3));
                    NewW(1,npx)=NewW(1,ordint2(1,4))*NewW(1,ordint(1,4));
                elseif p==5
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos,c_case+2*n+shockpos,c_case+3*n+shockpos,c_case+4*n+shockpos]; 
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos,c_case+2*n+statepos,c_case+3*n+statepos,c_case+4*n+statepos];               
                    NewX(1,npx-4)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx-3)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewX(1,npx-2)=NewX(1,ordint2(1,3))*NewX(1,ordint(1,3));
                    NewX(1,npx-1)=NewX(1,ordint2(1,4))*NewX(1,ordint(1,4));
                    NewX(1,npx)=NewX(1,ordint2(1,5))*NewX(1,ordint(1,5)); 
                    NewW(1,npx-4)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx-3)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                    NewW(1,npx-2)=NewW(1,ordint2(1,3))*NewW(1,ordint(1,3));
                    NewW(1,npx-1)=NewW(1,ordint2(1,4))*NewW(1,ordint(1,4));
                    NewW(1,npx)=NewW(1,ordint2(1,5))*NewW(1,ordint(1,5));          
                elseif p==6
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos,c_case+2*n+shockpos,c_case+3*n+shockpos,c_case+4*n+shockpos,c_case+5*n+shockpos]; 
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos,c_case+2*n+statepos,c_case+3*n+statepos,c_case+4*n+statepos,c_case+5*n+statepos];  
                    NewX(1,npx-5)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx-4)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewX(1,npx-3)=NewX(1,ordint2(1,3))*NewX(1,ordint(1,3));
                    NewX(1,npx-2)=NewX(1,ordint2(1,4))*NewX(1,ordint(1,4));
                    NewX(1,npx-1)=NewX(1,ordint2(1,5))*NewX(1,ordint(1,5));
                    NewX(1,npx)=NewX(1,ordint2(1,6))*NewX(1,ordint(1,6));
                    NewW(1,npx-5)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx-4)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                    NewW(1,npx-3)=NewW(1,ordint2(1,3))*NewW(1,ordint(1,3));
                    NewW(1,npx-2)=NewW(1,ordint2(1,4))*NewW(1,ordint(1,4));
                    NewW(1,npx-1)=NewW(1,ordint2(1,5))*NewW(1,ordint(1,5));
                    NewW(1,npx)=NewW(1,ordint2(1,6))*NewW(1,ordint(1,6));
                elseif p==7
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos,c_case+2*n+shockpos,c_case+3*n+shockpos,c_case+4*n+shockpos,c_case+5*n+shockpos,c_case+6*n+shockpos]; 
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos,c_case+2*n+statepos,c_case+3*n+statepos,c_case+4*n+statepos,c_case+5*n+statepos,c_case+6*n+statepos];  
                    NewX(1,npx-6)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx-5)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewX(1,npx-4)=NewX(1,ordint2(1,3))*NewX(1,ordint(1,3));
                    NewX(1,npx-3)=NewX(1,ordint2(1,4))*NewX(1,ordint(1,4));
                    NewX(1,npx-2)=NewX(1,ordint2(1,5))*NewX(1,ordint(1,5));
                    NewX(1,npx-1)=NewX(1,ordint2(1,6))*NewX(1,ordint(1,6));
                    NewX(1,npx)=NewX(1,ordint2(1,7))*NewX(1,ordint(1,7));
                    NewW(1,npx-6)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx-5)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                    NewW(1,npx-4)=NewW(1,ordint2(1,3))*NewW(1,ordint(1,3));
                    NewW(1,npx-3)=NewW(1,ordint2(1,4))*NewW(1,ordint(1,4));
                    NewW(1,npx-2)=NewW(1,ordint2(1,5))*NewW(1,ordint(1,5));
                    NewW(1,npx-1)=NewW(1,ordint2(1,6))*NewW(1,ordint(1,6));
                    NewW(1,npx)=NewW(1,ordint2(1,7))*NewW(1,ordint(1,7));                
                elseif p==8
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos,c_case+2*n+shockpos,c_case+3*n+shockpos,c_case+4*n+shockpos,c_case+5*n+shockpos,c_case+6*n+shockpos,c_case+7*n+shockpos]; 
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos,c_case+2*n+statepos,c_case+3*n+statepos,c_case+4*n+statepos,c_case+5*n+statepos,c_case+6*n+statepos,c_case+7*n+shockpos];  
                    NewX(1,npx-7)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx-6)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewX(1,npx-5)=NewX(1,ordint2(1,3))*NewX(1,ordint(1,3));
                    NewX(1,npx-4)=NewX(1,ordint2(1,4))*NewX(1,ordint(1,4));
                    NewX(1,npx-3)=NewX(1,ordint2(1,5))*NewX(1,ordint(1,5));
                    NewX(1,npx-2)=NewX(1,ordint2(1,6))*NewX(1,ordint(1,6));
                    NewX(1,npx-1)=NewX(1,ordint2(1,7))*NewX(1,ordint(1,7));
                    NewX(1,npx)=NewX(1,ordint2(1,8))*NewX(1,ordint(1,8));
                    NewW(1,npx-7)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx-6)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                    NewW(1,npx-5)=NewW(1,ordint2(1,3))*NewW(1,ordint(1,3));
                    NewW(1,npx-4)=NewW(1,ordint2(1,4))*NewW(1,ordint(1,4));
                    NewW(1,npx-3)=NewW(1,ordint2(1,5))*NewW(1,ordint(1,5));
                    NewW(1,npx-2)=NewW(1,ordint2(1,6))*NewW(1,ordint(1,6));
                    NewW(1,npx-1)=NewW(1,ordint2(1,7))*NewW(1,ordint(1,7));
                    NewW(1,npx)=NewW(1,ordint2(1,8))*NewW(1,ordint(1,8));        
                elseif p==9
                    ordint(1,:)=[c_case+shockpos,c_case+n+shockpos,c_case+2*n+shockpos,c_case+3*n+shockpos,c_case+4*n+shockpos,c_case+5*n+shockpos,c_case+6*n+shockpos,c_case+7*n+shockpos,c_case+8*n+shockpos]; 
                    ordint2(1,:)=[c_case+statepos,c_case+n+statepos,c_case+2*n+statepos,c_case+3*n+statepos,c_case+4*n+statepos,c_case+5*n+statepos,c_case+6*n+statepos,c_case+7*n+shockpos,c_case+8*n+shockpos];  
                    NewX(1,npx-8)=NewX(1,ordint2(1,1))*NewX(1,ordint(1,1));
                    NewX(1,npx-7)=NewX(1,ordint2(1,2))*NewX(1,ordint(1,2));
                    NewX(1,npx-6)=NewX(1,ordint2(1,3))*NewX(1,ordint(1,3));
                    NewX(1,npx-5)=NewX(1,ordint2(1,4))*NewX(1,ordint(1,4));
                    NewX(1,npx-4)=NewX(1,ordint2(1,5))*NewX(1,ordint(1,5));
                    NewX(1,npx-3)=NewX(1,ordint2(1,6))*NewX(1,ordint(1,6));
                    NewX(1,npx-2)=NewX(1,ordint2(1,7))*NewX(1,ordint(1,7));
                    NewX(1,npx-1)=NewX(1,ordint2(1,8))*NewX(1,ordint(1,8));
                    NewX(1,npx)=NewX(1,ordint2(1,9))*NewX(1,ordint(1,9));
                    NewW(1,npx-8)=NewW(1,ordint2(1,1))*NewW(1,ordint(1,1));
                    NewW(1,npx-7)=NewW(1,ordint2(1,2))*NewW(1,ordint(1,2));
                    NewW(1,npx-6)=NewW(1,ordint2(1,3))*NewW(1,ordint(1,3));
                    NewW(1,npx-5)=NewW(1,ordint2(1,4))*NewW(1,ordint(1,4));
                    NewW(1,npx-4)=NewW(1,ordint2(1,5))*NewW(1,ordint(1,5));
                    NewW(1,npx-3)=NewW(1,ordint2(1,6))*NewW(1,ordint(1,6));
                    NewW(1,npx-2)=NewW(1,ordint2(1,7))*NewW(1,ordint(1,7));
                    NewW(1,npx-1)=NewW(1,ordint2(1,8))*NewW(1,ordint(1,8));
                    NewW(1,npx)=NewW(1,ordint2(1,9))*NewW(1,ordint(1,9));           
                end       
            end
                
            X=NewX;
            W=NewW;
                
            Y(tt+1,:)=X*A+u(1+tt,:); %next period's endogenous variables
            Z(tt+1,:)=W*A+u(1+tt,:);
            irf(tt+1,:,ic)=Y(tt+1,:)-Z(tt+1,:); %store the empirical response fot the period tt
  
        end
        
        % repeat the draw if residuals extracted make the GIRF explosive 
        if irf(h,shockpos,ic)<=S(shockpos,shockpos)*10*abs(shock) && irf(h,shockpos,ic)>=-S(shockpos,shockpos)*10*abs(shock);      
            ic=ic+1; % not explosive: move to next draw 
        else %the draw is repeated since ic remain the same...
            nrep(1,ii)=nrep(1,ii)+1;
            if nrep(1,ii)==iter/2 % ...but if the same extraction has
                %been repeated for much of the iter considered
                %it means that is not only due to a particular sequence
                %of future shocks, but rather that the starting history
                %considered is explosive in itself
                nexplos=nexplos+1;
                irf=zeros(h,n,iter); % to not count at all when computing the average
                break % to leave the while loop and consider the next history
            end
        end
        
    end % end loop over iterations
    
    IRFhist(:,:,ii)=mean(irf,3); % mean across iterations of a particular history
    
end
if printprogress
    fprintf('\n');
    disp(['Number of GIRFs that are explosive: ' num2str(nexplos) '/' num2str(nhist) ] )
end

%% Average the mean response across (non-explosive) histories
IRFhistsum=zeros(h,n);
for hh=1:nhist
    IRFhistsum=IRFhistsum+IRFhist(:,:,hh);
end
GIRF=IRFhistsum/(nhist-nexplos); % sample mean across the non-explosive IRF


%% Output
GIRFopt.S = S;
GIRFopt.h = h;
GIRFopt.statepos = statepos;
GIRFopt.shockpos = shockpos;
GIRFopt.impulse = impulse;
GIRFopt.nexplos = nexplos;
GIRFopt.nhist = nhist;
GIRFopt.nrep = nrep;
