function [STVAR] = estimateSTVAR(data, state, shockpos, shocksize, p, c_case, exdata, h, MCMCn, alpha)

% The middle part of this function is largely based on publicly available
% code from Auerbach & Gorodnichenko (2012, 13).

% Prep
t = size(data, 1);
n = size(data, 2);
s = [NaN; state.s(1:end-1)]; % lagged by one period
Fs = [NaN; state.Fs(1:end-1)];

%% Y matrix
Y = data;

%% X matrix: Generate lags
X = [];
for pp = 1:p
    X = [X, [NaN*ones(pp, n); data(1:end-pp,:)] ];
end
% lags introduce some NA's => truncate
X = X(p+1:end,:); 
Y = Y(p+1:end,:);
s = s(p+1:end,:);
Fs = Fs(p+1:end,:);
% add constant and/or trend to X matrix
if c_case == 0
    c = [];
elseif c_case == 1
    c = ones(size(X,1),1); 
elseif c_case == 2
    c = [ones(size(X,1),1), [1:1:size(X)]' ];
else
    print('c_case variable needs to be set to 0, 1, or 2.')
end
% add exogenous variables
if ~isempty(exdata)
    Xex = [];
    for pp = 1:p
        Xex = [Xex, [NaN*ones(pp, size(exdata,2)); exdata(1:end-pp,:)]  ];
    end
    Xex = Xex(p+1:end,:);
else
    Xex = [];
end

%% Linear and nonlinear VAR for initial values
[rX, cX] = size(X);

% Nonlinear
Xm=[ (1-Fs).*X, Fs.*X ones(length(Y),1) Xex];
A=inv(Xm'*Xm)*(Xm'*Y);
u=Y-Xm*A;
Omega=cov(u);

AIC=log(det(Omega))+2/length(u)*length(A); % lag selection criteria
BIC=log(det(Omega))+log(length(u))/length(u)*length(A);

% linear benchmark
Xm1=[X ones(length(Y),1) Xex];
Alin=inv(Xm1'*Xm1)*(Xm1'*Y);
ulin=Y-Xm1*Alin;
Omegalin=cov(ulin);
Alin=Alin(1:end-1-size(Xex,2),:);

AIClin=log(det(Omegalin))+2/length(u)*length(Alin);
BIClin=log(det(Omegalin))+log(length(u))/length(u)*length(Alin);

disp('Lag length criteria')
disp('Non-linear model:')
disp(['  AIC =' num2str(AIC)])
disp(['  BIC =' num2str(BIC)])
disp('Linear model')
disp(['  AIC =' num2str(AIClin)])
disp(['  BIC =' num2str(BIClin)])
clear AIC BIC AIClin BIClin

%% 'Translate' variables to the way used by A&G
% data container
s_data.Y0 = Y;
s_data.X = X;
s_data.Z0 = s;
s_exo.FX = Xex;
s_exo.Nforecs = size(Xex,2);
% VAR properties
s_desc.VARlags=p;      % number of lags in the VAR
s_desc.Nvar=n;                  % number of variables in the VAR
s_desc.shockpos = shockpos; % position of government spending in the vector of variables in the VAR
s_desc.T = length(Y);
if c_case == 0
    s_exo.trendON = 0; %0 = trend, 1 = no trend
    s_exo.inter_regspec = 0; % 1=regime-specific intercept
    s_exo.trend_regspec = 0; % 1= regime-specific trend
elseif c_case == 1
    s_exo.trendON = 0;
    s_exo.inter_regspec = 1;
    s_exo.trend_regspec = 0; 
elseif c_case == 2
    s_exo.trendON = 1;
    s_exo.inter_regspec = 1;
    s_exo.trend_regspec = 1;
end
% Transition function
s_prior.theta0              = -state.gamma;    % the slope in the switching equation
s_prior.scale_penalty       = 1e6;  % gamma calibrated with certainty
s_prior.scale_penalty_beta  = 0;
% Parameters of the Minnesota prior: random walk for each variable in the VAR
s_prior.lambda_prior=1;   % controls cross-variable responses (smaller values mean stronger prior
s_prior.theta_prior=0.5;  % controls own response of variables
s_MCMC.MCMCn=MCMCn;   % number of MCMC draws
s_MCMC.dropfirstT=round(s_MCMC.MCMCn*0.2); % burn-in period for MCMC

%% Initial values (maximum likelihood)
% generate minnesota prior
[s_prior.prior_mean,s_prior.prior_var]=genMNprior(s_desc.Nvar,s_desc.VARlags,Omegalin,s_prior.lambda_prior,s_prior.theta_prior);
% note that the MN prior does not provide any information for the
% covariance matrix of residuals. Hence, impose a flat prior "centered"
% around the OLS estimate
Dn=dupmat(s_desc.Nvar);
Dplus=inv(Dn'*Dn)*Dn';
Omegalin_var=2*Dplus*kron(Omegalin,Omegalin)*Dplus'/s_desc.T;
s_prior.OmegaVAR=1e6*ones(size(diag(Omegalin_var)));
s_prior.Omega=Omegalin;
% take a small departure for one of the regimes to make sure that starting
% values are slightly different
Omega1=Omega;
D=randn(length(Omega));
Omega2=Omega+min(eig(Omega))*D*D';
% Apply Cholesky decomposition since we need to look only for a lower
% trinagular matrix
S1=chol(Omega1)';
S2=chol(Omega2)';
s_desc.Omega_length_cov = length(vech(S1));
% initial parameter values stacked
theta_init=[vech(S2);  vech(S2);  s_prior.theta0];
itermax=1000;
options=optimset('MaxFunEvals',itermax,'MaxIter',itermax,'Display','off','TolFun',1e-6,'TolX',1e-6);
% fminunc requires Optimization Toolbox; choose (A) if available, else (B)
% (A)
%[theta,mleLcov,flagquasi]=fminunc(@max_LSTVAR_mleScov,theta_init,options,u,s_data.Z0,s_desc.Omega_length_cov,s_desc.T,s_desc.Nvar,s_prior.theta0,s_prior.scale_penalty);
% (B)
theta = theta_init;
% end
[theta,mleLcov,flagquasi]=fminsearch(@max_LSTVAR_mleScov,theta,options,u,s_data.Z0,s_desc.Omega_length_cov,s_desc.T,s_desc.Nvar,s_prior.theta0,s_prior.scale_penalty);
% convert vectorized matrices with parameters into appropriate matrices
[Omega1,Omega2,gamma]=vec2matScov(theta,s_desc.Omega_length_cov);
S1=chol(Omega1)';
S2=chol(Omega2)';

%% MCMC algorithm: 
% In: Data, likelihood function, prior 
% Out: Estimates for Omega1, Omega2, A1, A2, summarized in Theta
disp('Markov chain Monte Carlo...')
theta=[vech(S1);  vech(S2);  gamma];

[Omega1a2,A1mat,A2mat,acceptrate,valJ]=...
    optimizeMCMC_struct(theta,s_data,s_desc,s_prior,s_exo,s_MCMC);

% disregard first batch of posteriors
Omega1a2 = Omega1a2(s_MCMC.dropfirstT:end-1,:); % need to check if chain (along columns) has converged, e.g. plot(Omega1a2(:,1))
postlength = size(Omega1a2,1);
A1mat = A1mat(s_MCMC.dropfirstT:end-1,:);
A2mat = A2mat(s_MCMC.dropfirstT:end-1,:);
Omega1mat=Omega1a2(:, 1:s_desc.Omega_length_cov);
Omega2mat=Omega1a2(:, s_desc.Omega_length_cov+1:2*s_desc.Omega_length_cov);

% mean estimates of remaining draws
[Omega1,Omega2]=vec2matScov(mean(Omega1a2)',s_desc.Omega_length_cov);
S1=chol(Omega1)';
S2=chol(Omega2)';
A1=unvec(mean(A1mat)', n, n*p);
A2=unvec(mean(A2mat)', n, n*p);

%% Identified impulse response functions
% independent of estimation and therefore potentially replaceable with IV
% identification
IRF1b = NaN*ones(h, n, postlength);
IRF2b = NaN*ones(h, n, postlength);
disp('Constructing impulse responses...')
disp('Progress \n')
for bb = 1:postlength % for each draw and state, compute impulse response
    
    % needed in general (for identification mostly)
    [VARb.Omega1, VARb.Omega2] = vec2matScov(Omega1a2(bb,:)',s_desc.Omega_length_cov);  
    VARb.S1 = chol(VARb.Omega1)';
    VARb.S2 = chol(VARb.Omega2)';
    
    % for impulse responses in state 1
    VARb.p = s_desc.VARlags;
    VARb.n = n;
    VARb.A = unvec( A1mat(bb,:)', n, n*p)';
    VARb.c_case = 0;  % parameters for constant not in A&G code included
    VARb.C1 = dyn_multipliers(VARb, h);
    % identification
    VARb.shock = zeros(n,1);
    VARb.shock(s_desc.shockpos) = 1;
    % (A) option to rescaling % if shocksize ~= 0  % absolute values, e.g. 25bp = 0.25
    %    VARb.shock = VARb.shock ./ VARb.S1(s_desc.shockpos, s_desc.shockpos) .* shocksize;
    %end
    for hh=1:h
        IRF1b(hh,:,bb) = (VARb.C1(:,:,hh)*VARb.S1*VARb.shock)';
    end
    
    % for impulse responses in state 2
    VARb.A = unvec( A2mat(bb,:)', n, n*p)';
    VARb.C2 = dyn_multipliers(VARb, h);
    % identification
    VARb.shock = zeros(n,1);
    VARb.shock(s_desc.shockpos) = 1;
    % (A) if shocksize ~= 0  % absolute values, e.g. 25bp = 0.25
    %    VARb.shock = VARb.shock ./ VARb.S2(s_desc.shockpos, s_desc.shockpos) .* shocksize;
    %end
    for hh=1:h
        IRF2b(hh,:,bb) = (VARb.C2(:,:,hh)*VARb.S2*VARb.shock)';
    end
    
    fprintf(1,'\b\b\b\b%3.0f%%',100*(bb/postlength)); 
    
end
fprintf('\n');

% define upper and lower thresholds
if isempty(alpha)
    alpha = 90;
end
lo = (100-alpha)/2;
up = 100 - lo;

% read percentiles of IRF1/2b
IRF1 = mean(IRF1b, 3);
IRF1bands=prctile(IRF1b, [lo up],3);
IRF2 = mean(IRF2b, 3);
IRF2bands=prctile(IRF2b, [lo up],3);

% option to rescaling
if shocksize ~= 0
    IRF1bands = IRF1bands ./ IRF1(1,shockpos) * shocksize;
    IRF1 = IRF1 ./ IRF1(1,shockpos) * shocksize;
    IRF2bands = IRF2bands ./ IRF2(1,shockpos) * shocksize;
    IRF2 = IRF2 ./ IRF2(1,shockpos) * shocksize;
end

  
%% Housekeeping
STVAR.data = data;
STVAR.X = X;
STVAR.Xex = Xex;
STVAR.c_case = c_case;
STVAR.p = p;
STVAR.t = t;
STVAR.n = n;
STVAR.A1 = A1;
STVAR.A2 = A2;
STVAR.u = u;
STVAR.Omega1 = Omega1;
STVAR.Omega2 = Omega2;
STVAR.S1 = S1;
STVAR.S2 = S2;
STVAR.IRF1 = IRF1;
STVAR.IRF2 = IRF2;
STVAR.IRF1bands = IRF1bands;
STVAR.IRF2bands = IRF2bands;
STVAR.nonlinear = state.nonlinear;
STVAR.logistic = state.logistic;
STVAR.statevar = state.statevar;
STVAR.gamma = state.gamma;
STVAR.cq = state.cq;
STVAR.s = state.s;
STVAR.Fs = state.Fs;

