function [VAR] = estimateVAR_covid(data, p, c_case, exdata, Tcovid)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function computes the MLE parameter estimates of a VAR,
% augmented with a change in volatility at the time of Covid (March 2020).
%
%
%
% Tcovid:   observation corresponding to March 2020 (for example, if the
%           sample ends in May 2020, and the VAR is monthly, Tcovid should
%           be set to length(y)-2)
%
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Prep
t = size(data, 1);
n = size(data, 2);

%% Y matrix
Y = data;

%% X matrix: Generate lags
X = [];
for pp = 1:p
    X = [X, [NaN*ones(pp, n); data(1:end-pp,:)] ];
end
% lags introduce some NA's => truncate
X = X(p+1:end,:); 
Y = Y(p+1:end,:);
% add constant and/or trend to X matrix
if c_case == 0
    c = [];
elseif c_case == 1
    c = ones(size(X,1),1); 
elseif c_case == 2
    c = [ones(size(X,1),1), [1:1:size(X)]' ];
else
    print('c_case variable needs to be set to 0, 1, or 2.')
end
% add exogenous variables
if ~isempty(exdata)
    Xex = [];
    for pp = 1:p
        Xex = [Xex, [NaN*ones(pp, size(exdata,2)); exdata(1:end-pp,:)]  ];
    end
    Xex = Xex(p+1:end,:);
else
    Xex = [];
end
X = [c, X, Xex];

[T,n]=size(Y);
Tcovid=Tcovid-p;

% %% data matrix manipulations
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% % dimensions
% [TT,n]=size(y);
% k=n*p+1;         % # coefficients for each equation
% 
% 
% % constructing the matrix of regressors
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% x=zeros(TT,k);
% x(:,1)=1;
% for i=1:p
%     x(:,1+(i-1)*n+1:1+i*n)=lag(y,i);
% end
% x=x(p+1:end,:);
% y=y(p+1:end,:);



%% starting values and bounds for the minimization of -logLH
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
aux=mean(abs(Y(Tcovid:max([Tcovid+1 T]),:)-Y(Tcovid-1:max([Tcovid+1 T])-1,:))',1)./...
    mean(mean(abs(Y(2:Tcovid-1,:)-Y(1:Tcovid-2,:))));
if isempty(aux)
    eta0=[];
elseif length(aux)==2;
    eta0=[aux';aux(1);.8];      % volatility hyperparameters 
elseif length(aux)>=3;
    eta0=[aux(1:3)';.8];        % volatility hyperparameters 
end

% bounds
MIN.eta(1:3)=[1;1;1];
MAX.eta(1:3)=[500;500;500];
MIN.eta(4)=.005;
MAX.eta(4)=.995;

ncp=length(eta0);           % # "covid" hyperparameters
ineta=-log((MAX.eta'-eta0)./(eta0-MIN.eta'));
inHeta=(1./(MAX.eta'-eta0)+1./(eta0-MIN.eta')).^2.*(abs(eta0)/1).^2;

x0=[ineta];                 % initial guess for volatility hyperparameters
H0=diag([inHeta]);          % initial guess for the inverse Hessian

%% maximization of the concentrated LH with respect to the covid hyperparameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[fh,xh,gh,H,itct,fcount,retcodeh] = csminwel('loglhVAR_covid',x0,H0,[],1e-16,1000,Y,X,p,T,n,MIN,MAX,Tcovid);


%% output of the maximization
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% MLE of volatility parameters, VAR coefficients and residual covariance
[fh, A, u, Omega, S]=loglhVAR_covid(xh,Y,X,p,T,n,MIN,MAX,Tcovid);
%VAR.logLH=-fh;
%VAR.vol=MIN.eta'+(MAX.eta'-MIN.eta')./(1+exp(-xh));

%%Housekeeping
VAR.data = data;
VAR.exdata = exdata;
VAR.X = X(:,(c_case+1):(c_case+n*p));
VAR.Xex = Xex;
VAR.c_case = c_case;
VAR.p = p;
VAR.t = t;
VAR.n = n;
VAR.A = A;
VAR.u = u;
VAR.Omega = Omega;
VAR.S = S;

